
import * as sch from '../utilities/json-schema';
import { FastifyInstance, RouteShorthandOptions } from 'fastify';
import { HttpConfig, ServerStatus } from '../http/server';

export interface Dependencies {
	ready_check: () => ServerStatus;
}

export function register_ready_check_endpoint(http_server: FastifyInstance, conf: HttpConfig, { ready_check }: Dependencies) {
	const opts: RouteShorthandOptions = {
		schema: {
			response: {
				200: sch.obj({ status: sch.str_enum([ 'ready' ]) }),
				503: sch.obj({ status: sch.str_enum([ 'unstarted', 'starting', 'closing', 'closed' ]) }),
			}
		}
	};
	
	http_server.get('/ready', opts, async (req, res) => {
		const status = ready_check();
		res.status(status === 'ready' ? 200 : 503);
		return { status };
	});
}
