
import { pino } from 'pino';
import { Req } from './http/request';

export interface LoggingConfig {
	level: 'silent' | 'fatal' | 'error' | 'warn' | 'info' | 'debug' | 'trace';
	pretty: boolean;
}

export function validate_logging_conf(conf: unknown) : asserts conf is LoggingConfig {
	// todo: validate config
}

export function create_logger(conf: LoggingConfig) {
	const opts: pino.LoggerOptions = {
		level: conf.level,
		serializers: {
			req(req: Req) {
				const req_url = req.url.includes('/login-callback?')
					? `${req.url.split('/login-callback?')[0]}/login-callback?[redacted]`
					: req.url;

				return {
					method: req.method,
					url: req_url,
					hostname: req.hostname,
					remoteAddress: req.ip,
					remotePort: req.socket.remotePort,
					session: req.session?.key?.prefix,
				};
			}
		}
	};

	if (conf.pretty) {
		opts.transport = {
			target: 'pino-pretty',
			options: {
				colorize: true,
				translateTime: 'HH:MM:ss Z',
				ignore: 'pid,hostname'
			}
		}
	}

	return pino(opts);
}
