
import { hash, verify, argon2id } from 'argon2';

export interface Argon2HashConfig {
	/**  */
	hash_length: number;
	
	/**  */
	time_cost: number;
	
	/**  */
	memory_cost: number;

	/**  */
	parallelism: number;
}

export type Argon2HashProvider = ReturnType<typeof create_argon_hash_provider>;

export function create_argon_hash_provider(conf: Argon2HashConfig) {
	return {
		hash(password: string) {
			return hash(password, {
				type: argon2id,
				hashLength: conf.hash_length,
				timeCost: conf.time_cost,
				memoryCost: conf.memory_cost,
				parallelism: conf.parallelism,
			});
		},	
		verify(password: string, hash: string) {
			return verify(hash, password, {
				// 
			});
		},
	};
}
