
import { createHash } from 'crypto';
import { rand } from '../utilities/rand';
import { invalidate_cookie, parse_req_cookies, set_cookie } from '../http/cookies';
import { FastifyReply, FastifyRequest } from 'fastify';

export interface PKCECookieConfig {
	name: string;
	secure: boolean;
	ttl: number;
	code_bytes: number;
}

export function validate_pkce_cookie_conf(conf: unknown) : asserts conf is PKCECookieConfig {
	// todo: validate config
}

export type PKCECookieProvider = ReturnType<typeof create_pkce_cookie_provider>;

export function create_pkce_cookie_provider(conf: PKCECookieConfig) {
	return {
		async setup_pkce_challenge(res: FastifyReply) {
			const pkce_code_verifier = await rand(conf.code_bytes, 'base64url');
			const pkce_code_challenge = sha256_base64_url(pkce_code_verifier);
			const pkce_expire = new Date(Date.now() + (conf.ttl * 1000));
			
			// "Lax" rather than "Strict", so the PKCE verifier will be included on the
			// redirect to the login callback endpoint, which comes from the OpenID server,
			// which is likely on a different site
			set_cookie(res, conf.name, pkce_code_verifier, pkce_expire, conf.secure, 'Lax');
			
			return pkce_code_challenge;
		},

		read_pkce_code_verifier(req: FastifyRequest) {
			const cookies = parse_req_cookies(req);
			return cookies[conf.name];
		},

		reset(res: FastifyReply) {
			invalidate_cookie(res, conf.name, conf.secure);
		},
	}
}

function sha256_base64_url(input: string) {
	const hash = createHash('sha256');
	hash.update(input);
	return hash.digest('base64url');
}
