
import { pino } from 'pino';
import { make_data_dir } from '../files';
import { StorageProvider } from '../provider';
import { FileStorageConfig } from './config';
import { create_session, get_session, delete_session, cleanup_old_sessions } from './sessions';
import { get_user, create_user, update_user, delete_user, get_user_by_oidc_sub } from './users';

export function create_file_storage_provider(conf: FileStorageConfig, logger: pino.Logger) : StorageProvider {
	let status: 'ok' | 'warning' | 'updating' | 'unavailable' = 'unavailable';

	const init_steps = [
		make_data_dir('sessions'),
		make_data_dir('users'),
	];

	const ready = Promise.all(init_steps).then(() => {
		status = 'ok';
	});

	return {
		ready,

		status() {
			return {
				engine: 'file',
				status: status,
			};
		},
		
		// Login Sessions
		get_session,
		create_session,
		delete_session,
		cleanup_old_sessions,

		// Users
		get_user,
		get_user_by_oidc_sub,
		create_user,
		update_user,
		delete_user,
	};
}
