
import type { UserData } from '../provider';
import type { Snowflake } from '../../utilities/snowflake-uid';
import { delete_data_file, read_data_file, write_data_file } from '../files';
import { memo } from '../../utilities/memo';
import { HttpURL, Locale, Timezone } from '../../utilities/types';

export async function get_user(user_id: Snowflake) : Promise<UserData> {
	const data = await read_data_file<UserJson>(`users/${user_id}`, 'json');
	return from_json(data);
}

export async function get_user_by_oidc_sub(sub: string) : Promise<UserData> {
	// todo: get_user_by_oidc_sub
	return null;
}

export async function create_user(data: UserData) {
	const json = JSON.stringify(to_json(data), null, '  ');
	await write_data_file(`users/${data.id}`, json, 'wx');
}

export async function update_user(user_id: Snowflake, data: UserData) {
	// todo: update_user
}

export async function delete_user(user_id: Snowflake) {
	await delete_data_file(`users/${user_id}`);
	// todo: other cleanup
}

interface UserJson {
	id: Snowflake;
	username: string;
	oidc_subject?: string;
	name?: string;
	website?: HttpURL;
	profile?: HttpURL;
	picture?: HttpURL;
	locale: Locale;
	time_zone: Timezone;
}

function to_json(user: UserData) : UserJson {
	return {
		id: user.id,
		username: user.username,
		oidc_subject: user.oidc_subject,
		name: user.name,
		locale: user.locale,
		time_zone: user.timezone,
		picture: user.picture,
		profile: user.profile,
		website: user.website,
	};
}

function from_json(json: UserJson) : UserData {
	return {
		id: json.id,
		username: json.username,
		oidc_subject: json.oidc_subject,
		name: json.name,
		locale: json.locale,
		timezone: json.time_zone,
		picture: json.picture,
		profile: json.profile,
		website: json.website,
	};
}
