
import { promises as fs } from 'fs';
import { join as path_join } from 'path';

export const data_dir = process.env.DATA_PATH;

if (! data_dir) {
	console.error('No DATA_PATH defined');
	process.exit(1);
}

export function data_path(file: string) {
	return path_join(data_dir, file);
}

export async function read_data_file(file: string, encoding: 'binary') : Promise<Buffer>;
export async function read_data_file(file: string, encoding: 'text') : Promise<string>;
export async function read_data_file<T = unknown>(file: string, encoding: 'json') : Promise<T>;
export async function read_data_file(file: string, encoding: 'binary' | 'text' | 'json') {
	const path = data_path(file);

	let text: string;

	try {
		if (encoding === 'binary') {
			return await fs.readFile(path);
		}

		text = await fs.readFile(path, 'utf8');
	}

	catch (error) {
		// todo: handle read errors
	}

	switch (encoding) {
		case 'text': return text;
		case 'json': return JSON.parse(text);
	}
}

export async function write_data_file(file: string, content: string | Buffer, flag: 'w' | 'wx' = 'w') {
	const path = data_path(file);
	await fs.writeFile(path, content, {
		encoding: typeof content === 'string' ? 'utf8' : 'binary',
		flag
	});
}

export async function delete_data_file(file: string) {
	const path = data_path(file);
	await fs.unlink(path);
}

export async function make_data_dir(dir: string) {
	const path = data_path(dir);
	await fs.mkdir(path, {
		mode: 0o700,
		recursive: true,
	});
}

export function read_data_dir(dir: string) {
	const path = data_path(dir);
	return fs.readdir(path);
}
