
import type { Snowflake } from '../utilities/snowflake-uid';
import { HttpURL, Locale, Timezone } from '../utilities/types';

export interface StorageProvider {
	readonly ready: Promise<void | void[]>;

	status() : StorageStatus | Promise<StorageStatus>;

	// Login Sessions
	get_session(prefix: string) : Promise<SessionData>;
	create_session(data: SessionData) : Promise<void>;
	delete_session(prefix: string) : Promise<void>;
	cleanup_old_sessions() : Promise<void>;

	// Users
	get_user(id: Snowflake) : Promise<UserData>;
	get_user_by_oidc_sub(sub: string) : Promise<UserData>;
	create_user(data: UserData) : Promise<void>;
	update_user(id: Snowflake, data: UserData) : Promise<void>;
	delete_user(id: Snowflake) : Promise<void>;

	// todo: fill in with app-data-specific methods
}

export interface StorageStatus {
	engine: 'file' | 'sqlite3' | 'mysql';
	status: 'ok' | 'warning' | 'updating' | 'unavailable';
}

export interface SessionData {
	prefix: string;
	key_hash: string;
	user_id: Snowflake;
	started: Date;
	expires: Date;
}

export interface UserData {
	id: Snowflake;
	username: string;
	oidc_subject?: string;
	name?: string;
	website?: HttpURL;
	profile?: HttpURL;
	picture?: HttpURL;
	locale: Locale;
	timezone: Timezone;
}
