
import { run } from './run';
import type { DB } from './db';
import type * as sqlite3 from 'sqlite3';

export const type = {
	rowid: 'integer' as const,
	url: 'varchar(1000)' as const,
	snowflake: 'integer' as const,
	uuid: 'varchar(50)' as const,
	ref_value: 'varchar(64)' as const,
	bool: 'tinyint unsigned' as const,
};

export async function create_ref_table(db: DB, conn: sqlite3.Database, table_name: string, values: string[]) {
	const sql_create_table = `
		create table if not exists ${table_name} (
			value ${type.ref_value} not null primary key
		)
	`;

	const sql_insert_values = `
		insert into ${table_name}
			(value)
		values ${values.map(() => `(?)`).join(', ')}
	`;

	await run(db, conn, sql_create_table);
	await run(db, conn, sql_insert_values, values);
}
