
import { run } from '../run';
import { type } from '../schema-utils';
import type { DB } from '../db';
import type * as sqlite3 from 'sqlite3';
import type { Snowflake } from '../../../utilities/snowflake-uid';

export async function migrate_v0_to_v1(db: DB) {
	await Promise.all([
		// migrate_v1_settings(db),
		migrate_v1_users(db),
		migrate_v1_sessions(db),
	]);
}



// ===== Settings Database =====

async function migrate_v1_settings(db: DB) {
	const conn = await db.settings.acquire();

	try {
		await create_table_settings(db, conn);
	}

	finally {
		await db.settings.release(conn);
	}
}

const sql_create_table_settings = `
create table if not exists settings (
	// 
)`;

async function create_table_settings(db: DB, conn: sqlite3.Database) {
	await run(db, conn, sql_create_table_settings);
}



// ===== Users Database =====

async function migrate_v1_users(db: DB) {
	const conn = await db.users.acquire();

	try {
		await create_table_users(db, conn);
	}

	finally {
		await db.users.release(conn);
	}
}

const sql_create_table_users = `
create table if not exists users (
	id ${type.rowid} primary key,
	username text not null,
	oidc_subject text not null,
	name text,
	locale varchar(20) not null default 'en-US',
	timezone text not null default 'Africa/Abidjan',
	website ${type.url},
	profile ${type.url},
	picture ${type.url},

	unique(username),
	unique(oidc_subject)
)`;

async function create_table_users(db: DB, conn: sqlite3.Database) {
	await run(db, conn, sql_create_table_users);
}



// ===== Sessions Database =====

async function migrate_v1_sessions(db: DB) {
	const conn = await db.sessions.acquire();

	try {
		await create_table_sessions(db, conn);
	}

	finally {
		await db.sessions.release(conn);
	}
}

export interface SessionRow {
	prefix: string;
	key_hash: string;
	user_id: Snowflake;
	started: string;
	expires: string;
}

const sql_create_table_sessions = `
create table if not exists sessions (
	prefix varchar(16) not null primary key,
	key_hash text not null,
	user_id ${type.rowid},
	started datetime not null,
	expires datetime not null
)`;

async function create_table_sessions(db: DB, conn: sqlite3.Database) {
	await run(db, conn, sql_create_table_sessions);
}

