
import { randomBytes } from 'crypto';

export type BufferEncoded<T extends BufferEncoding> = T extends 'binary' ? Buffer : string;

export function rand<T extends BufferEncoding = 'binary'>(size: number, encoding?: T) : Promise<BufferEncoded<T>> {
	return new Promise<BufferEncoded<T>>((resolve, reject) => {
		randomBytes(size, (error, buffer) => {
			if (error) {
				return reject(error);
			}

			if (encoding && encoding !== 'binary') {
				resolve(buffer.toString(encoding) as BufferEncoded<T>);
				return;
			}

			resolve(buffer as BufferEncoded<T>);
		});
	});
}
